@@ -144,9 +144,10 @@ braced s = "{" ++ s ++ "}"
 inQuotes :: BibParser String
 inQuotes = do
   char '"'
-  concat <$> manyTill (try (string "\\\"")
-                     <|> many1 (noneOf "\"\\")
-                     <|> count 1 anyChar) (char '"')
+  concat <$> manyTill (  many1 (noneOf "\"\\{")
+                     <|> (char '\\' >> (\c -> ['\\',c]) <$> anyChar)
+                     <|> braced <$> inBraces
+                      ) (char '"')
 
 fieldName :: BibParser String
 fieldName = (map toLower) <$> many1 (letter <|> digit <|> oneOf "-_")
@@ -322,7 +323,7 @@ resolveKey lang (Formatted ils) = Formatted (Walk.walk go ils)
 
 resolveKey' :: Lang -> String -> String
 resolveKey' (Lang "en" "US") k =
-  case k of
+  case map toLower k of
        "inpreparation" -> "in preparation"
        "submitted"     -> "submitted"
        "forthcoming"   -> "forthcoming"
@@ -536,6 +537,7 @@ toAuthor _ [Str "others"] = return $
           , nameSuffix      = mempty
           , literal         = Formatted [Str "others"]
           , commaSuffix     = False
+          , parseNames      = True
           }
 toAuthor _ [Span ("",[],[]) ils] =
   return $ -- corporate author
@@ -546,6 +548,7 @@ toAuthor _ [Span ("",[],[]) ils] =
           , nameSuffix      = mempty
           , literal         = Formatted ils
           , commaSuffix     = False
+          , parseNames      = True
           }
 -- First von Last
 -- von Last, First
@@ -598,6 +601,7 @@ toAuthor opts ils = do
           , nameSuffix      = suffix
           , literal         = mempty
           , commaSuffix     = usecomma
+          , parseNames      = True
           }
 
 isCapitalized :: [Inline] -> Bool
